==> gcma-0.1.12.tar.gz: gcma-0.1.12/gcma/plotter/Plotter.py <==
import logging
from typing import Union

import numpy as np
import pandas as pd
import seaborn as sns
import treefiles as tf
from matplotlib import pyplot as plt
from matplotlib.ticker import MaxNLocator

from gcma import CMA
from gcma.io.utils import load_legends
from gcma.plotter._plot_movie import _plot_dist


class Plotter:
    plot_dist = _plot_dist

    def __init__(
        self,
        parent: CMA,
        save_dir: Union[tf.Tree, str] = None,
        show: bool = False,
        force_recompute: bool = True,
    ):
        self.parent = parent
        self.out_dir = parent.output_dir
        if isinstance(save_dir, str) or save_dir is None:
            self.save_dir = self.out_dir.dir("plots")
        else:
            self.save_dir = save_dir
        self.save_dir.dump()

        self.params, self.features = load_legends(self.out_dir.legends)

        self.show = show
        self.force_recompute = force_recompute

        log.debug(f"Ready to save figures to {self.save_dir.abs()}")

    def plot_params(self):
        params_fname = self.save_dir.path("params_pd.pkl")
        if tf.greedy_download(params_fname, force=self.force_recompute):
            params_history = np.load(self.out_dir.pa)
            cols = [x.name for x in self.params.values()]
            params_pd = get_df(params_history, cols, "p")
            params_pd.to_pickle(params_fname)
        else:
            log.debug(f"Not recomputing {params_fname}")
            params_pd = pd.read_pickle(params_fname)

        fname = self.save_dir.path("params_history.png")
        with tf.SPlot(fname, show=self.show):
            ax = sns.lineplot(data=params_pd, x=SB.iter, y=SB.val_p, hue=SB.var_p)
            ax.set_ylim(0, self.parent.scale_factor)
            ax.xaxis.set_major_locator(MaxNLocator(integer=True))
            ax.set_title("Parameters History")
            plt.legend(loc="upper left")
            sns.despine()

        log.info(f"Finished to plot {fname}")

    def plot_non_normalized_params(self, *names: str, y: str = None):
        all_names = self.params.keys()
        if len(names) == 0:
            names = list(all_names)

        if y is None:
            y = "Value"

        tmp = self.parent.rescale(np.load(self.out_dir.pa))
        params_history = np.empty(
            (self.parent.es.countiter, self.parent.popsize, len(names))
        )
        for i, n in enumerate(all_names):
            if n in names:
                params_history[..., names.index(n)] = tmp[..., i]

        params_pd = get_df(params_history, list(names), "p")
        params_pd = params_pd.rename(columns={SB.val_p: y})

        bd = self.parent.bounds
        a, b = bd[:, 0].min(), bd[:, 1].max()

        fname = self.save_dir.path("params_history_non_normalized.png")
        with tf.SPlot(fname, show=self.show):
            ax = sns.lineplot(data=params_pd, x=SB.iter, y=y, hue=SB.var_p)
            ax.xaxis.set_major_locator(MaxNLocator(integer=True))
            ax.set_ylim(a, b)
            ax.set_title("Parameters History")
            plt.legend(loc="upper left")
            sns.despine()

        log.info(f"Finished to plot {fname}")

    def plot_errors(self):
        energy_fname = self.save_dir.path("energy_pd.pkl")
        if tf.greedy_download(energy_fname, force=self.force_recompute):
            energy_history_ft = np.load(self.out_dir.enf, allow_pickle=True)
            energy_history_global = np.load(self.out_dir.en)

            for x in [("energy_history_ft", 3), ("energy_history_global", 2)]:
                var = locals()[x[0]]
                assert var.ndim == x[1], f"{x[0]} shape is {var.shape}"

            energy_history_global = energy_history_global[..., np.newaxis]
            energy_history = np.dstack((energy_history_ft, energy_history_global))
            cols = [x.name for x in self.features.values()]
            energy_pd = get_df(energy_history, cols + [SB.glo], "f")

            energy_pd.to_pickle(energy_fname)
        else:
            log.debug(f"Not recomputing {energy_fname}")
            energy_pd = pd.read_pickle(energy_fname)

        vals = energy_pd.loc[energy_pd[SB.var_f] == SB.glo][SB.val_f]
        vals = vals.values[np.where(vals < 1e3)]
        val = vals.mean() + vals.std()

        energy_pd[SB.val_f] = energy_pd[SB.val_f].astype(float)

        fname = self.save_dir.path("energy_history.png")
        with tf.SPlot(fname, show=self.show):
            ax = sns.lineplot(data=energy_pd, x=SB.iter, y=SB.val_f, hue=SB.var_f)
            ax.set_ylim(0, val)
            ax.xaxis.set_major_locator(MaxNLocator(integer=True))
            ax.set_title("Features Error")
            plt.legend(loc="upper left")
            sns.despine()

        log.info(f"Finished to plot {fname}")

    def plot_best_error(self):
        best_energy_fname = self.save_dir.path("best_energy_pd.pkl")
        if tf.greedy_download(best_energy_fname, force=self.force_recompute):
            energy_history_ft = np.load(self.out_dir.enf, allow_pickle=True)
            energy_history_global = np.load(self.out_dir.en)

            for x in [("energy_history_ft", 3), ("energy_history_global", 2)]:
                var = locals()[x[0]]
                assert var.ndim == x[1], f"{x[0]} shape is {var.shape}"

            energy_history_global = energy_history_global[..., np.newaxis]
            energy_history = np.dstack((energy_history_ft, energy_history_global))

            s = energy_history.shape
            best_errs = np.empty((s[0], 1, s[2]))
            for i in range(s[0]):
                ite = energy_history[i, ...]
                j = np.argmin(ite[:, 0])
                best_errs[i, 0, :] = energy_history[i, j, :]

            cols = [x.name for x in self.features.values()]
            best_pd = get_df(best_errs, cols + [SB.glo], "f")
            best_pd.to_pickle(best_energy_fname)
        else:
            log.debug(f"Not recomputing {best_energy_fname}")
            best_pd = pd.read_pickle(best_energy_fname)

        vals = best_pd.loc[best_pd[SB.var_f] == SB.glo][SB.val_f]
        vals = vals.values[np.where(vals < 1e3)]
        val = vals.mean() + 2 * vals.std()

        fname = self.save_dir.path("best_energy_history.png")
        with tf.SPlot(fname, show=self.show):
            ax = sns.lineplot(data=best_pd, x=SB.iter, y=SB.val_f, hue=SB.var_f)
            ax.set_ylim(0, val)
            ax.xaxis.set_major_locator(MaxNLocator(integer=True))
            ax.set_title("Best Features Error")
            plt.legend(loc="upper left")
            sns.despine()

        log.info(f"Finished to plot {fname}")


class SB:
    iter = "CMA-ES iteration"
    var_p = "Parameters"
    var_f = "Features"
    val_p = "Normalised value"
    val_f = "Error $L_2$-norm"
    glo = "Global Error"


def get_df(params, columns, idx):
    nb_iters, nb_pop, nb_params = params.shape
    p_arr = np.column_stack(
        (np.repeat(np.arange(nb_iters), nb_pop), params.reshape(nb_iters * nb_pop, -1))
    )
    p_df = pd.DataFrame(p_arr, columns=[SB.iter] + columns)
    p_df[SB.iter] = p_df[SB.iter].astype(int)

    p_df = pd.melt(
        p_df,
        id_vars=[SB.iter],
        value_vars=columns,
        var_name=getattr(SB, f"var_{idx}"),
        value_name=getattr(SB, f"val_{idx}"),
    )

    return p_df


log = logging.getLogger(__name__)

==> gcma-0.1.16.tar.gz: gcma-0.1.16/gcma/plotter/Plotter.py <==
import logging
from typing import Union

import numpy as np
import pandas as pd
import seaborn as sns
import treefiles as tf
from matplotlib import pyplot as plt
from matplotlib.ticker import MaxNLocator

from gcma import CMA
from gcma.io.utils import load_legends
from gcma.plotter._plot_movie import _plot_dist


class Plotter:
    plot_dist = _plot_dist

    def __init__(
        self,
        parent: CMA,
        save_dir: Union[tf.Tree, str] = None,
        show: bool = False,
        force_recompute: bool = True,
    ):
        self.parent = parent
        self.out_dir = parent.output_dir
        if isinstance(save_dir, str) or save_dir is None:
            self.save_dir = self.out_dir.dir("plots")
        else:
            self.save_dir = save_dir
        self.save_dir.dump()

        self.params, self.features = load_legends(self.out_dir.legends)
        self.pnames = [x.get_pname() for x in self.params.values()]
        self.fnames = [x.get_pname() for x in self.features.values()]

        self.show = show
        self.force_recompute = force_recompute

        log.debug(f"Ready to save figures to {self.save_dir.abs()}")

    def plot_params(self, fname: str = None, title: str = None):
        params_fname = self.save_dir.path("params_pd.pkl")
        if tf.greedy_download(params_fname, force=self.force_recompute):
            params_history = np.load(self.out_dir.pa)
            params_pd = get_df(params_history, self.pnames, "p")
            params_pd.to_pickle(params_fname)
        else:
            log.debug(f"Not recomputing {params_fname}")
            params_pd = pd.read_pickle(params_fname)

        if title is None:
            title = "Parameters History"
        if fname is None:
            fname = "params_history.png"
        fname = self.save_dir.path(fname)

        with tf.SPlot(fname, show=self.show):
            ax = sns.lineplot(data=params_pd, x=SB.iter, y=SB.val_p, hue=SB.var_p)
            ax.set_ylim(0, self.parent.scale_factor)
            ax.xaxis.set_major_locator(MaxNLocator(integer=True))
            ax.set_title(title)
            plt.legend(loc="upper left")
            sns.despine()

        log.info(f"Finished to plot {fname}")

    def plot_non_normalized_params(
        self, *names: str, y: str = None, title: str = None, fname: str = None
    ):
        all_names = self.pnames
        if len(names) == 0:
            bd = self.parent.bounds
            names = all_names
        else:
            bd = np.array([self.params[x].bounds for x in names])
            names = [self.params[x].get_pname() for x in names]
        a, b = bd[:, 0].min(), bd[:, 1].max()

        if y is None:
            y = "Value"

        tmp = self.parent.rescale(np.load(self.out_dir.pa))
        params_history = np.empty(
            (self.parent.es.countiter, self.parent.popsize, len(names))
        )
        for i, n in enumerate(all_names):
            if n in names:
                params_history[..., names.index(n)] = tmp[..., i]

        params_pd = get_df(params_history, list(names), "p")
        params_pd = params_pd.rename(columns={SB.val_p: y})

        if title is None:
            title = "Parameters History"
        if fname is None:
            fname = "params_history_non_normalized.png"
        fname = self.save_dir.path(fname)
        with tf.SPlot(fname, show=self.show):
            ax = sns.lineplot(data=params_pd, x=SB.iter, y=y, hue=SB.var_p)
            ax.xaxis.set_major_locator(MaxNLocator(integer=True))
            ax.set_ylim(a, b)
            ax.set_title(title)
            plt.legend(loc="upper left")
            sns.despine()

        log.info(f"Finished to plot {fname}")

    def plot_errors(self, title: str = None, fname: str = None):
        energy_fname = self.save_dir.path("energy_pd.pkl")
        if tf.greedy_download(energy_fname, force=self.force_recompute):
            energy_history_ft = np.load(self.out_dir.enf, allow_pickle=True)
            energy_history_global = np.load(self.out_dir.en)

            for x in [("energy_history_ft", 3), ("energy_history_global", 2)]:
                var = locals()[x[0]]
                assert var.ndim == x[1], f"{x[0]} shape is {var.shape}"

            energy_history_global = energy_history_global[..., np.newaxis]
            energy_history = np.dstack((energy_history_ft, energy_history_global))
            energy_pd = get_df(energy_history, self.fnames + [SB.glo], "f")

            energy_pd.to_pickle(energy_fname)
        else:
            log.debug(f"Not recomputing {energy_fname}")
            energy_pd = pd.read_pickle(energy_fname)

        vals = energy_pd.loc[energy_pd[SB.var_f] == SB.glo][SB.val_f]
        # vals = vals.values[np.where(vals < 1e3)]
        val = vals.mean() + vals.std()

        energy_pd[SB.val_f] = energy_pd[SB.val_f].astype(float)

        if title is None:
            title = "Features Error"
        if fname is None:
            fname = "energy_history.png"
        fname = self.save_dir.path(fname)
        with tf.SPlot(fname, show=self.show):
            ax = sns.lineplot(data=energy_pd, x=SB.iter, y=SB.val_f, hue=SB.var_f)
            ax.set_ylim(0, val)
            ax.xaxis.set_major_locator(MaxNLocator(integer=True))
            ax.set_title(title)
            plt.legend(loc="upper left")
            sns.despine()

        log.info(f"Finished to plot {fname}")

    def plot_best_error(self, title: str = None, fname: str = None):
        best_energy_fname = self.save_dir.path("best_energy_pd.pkl")
        if tf.greedy_download(best_energy_fname, force=self.force_recompute):
            energy_history_ft = np.load(self.out_dir.enf, allow_pickle=True)
            energy_history_global = np.load(self.out_dir.en)

            for x in [("energy_history_ft", 3), ("energy_history_global", 2)]:
                var = locals()[x[0]]
                assert var.ndim == x[1], f"{x[0]} shape is {var.shape}"

            energy_history_global = energy_history_global[..., np.newaxis]
            energy_history = np.dstack((energy_history_ft, energy_history_global))

            s = energy_history.shape
            best_errs = np.empty((s[0], 1, s[2]))
            for i in range(s[0]):
                ite = energy_history[i, ...]
                j = np.argmin(ite[:, 0])
                best_errs[i, 0, :] = energy_history[i, j, :]

            best_pd = get_df(best_errs, self.fnames + [SB.glo], "f")
            best_pd.to_pickle(best_energy_fname)
        else:
            log.debug(f"Not recomputing {best_energy_fname}")
            best_pd = pd.read_pickle(best_energy_fname)

        vals = best_pd.loc[best_pd[SB.var_f] == SB.glo][SB.val_f]
        # vals = vals.values[np.where(vals < 1e3)]
        val = vals.mean() + 2 * vals.std()

        if title is None:
            title = "Best Features Error"
        if fname is None:
            fname = "best_energy_history.png"
        fname = self.save_dir.path(fname)
        with tf.SPlot(fname, show=self.show):
            ax = sns.lineplot(data=best_pd, x=SB.iter, y=SB.val_f, hue=SB.var_f)
            ax.set_ylim(0, val)
            ax.xaxis.set_major_locator(MaxNLocator(integer=True))
            ax.set_title(title)
            plt.legend(loc="upper left")
            sns.despine()

        log.info(f"Finished to plot {fname}")


class SB:
    iter = "CMA-ES iteration"
    var_p = "Parameters"
    var_f = "Features"
    val_p = "Normalised value"
    val_f = "Error $L_2$-norm"
    glo = "Global Error"


def get_df(params, columns, idx):
    nb_iters, nb_pop, nb_params = params.shape
    p_arr = np.column_stack(
        (np.repeat(np.arange(nb_iters), nb_pop), params.reshape(nb_iters * nb_pop, -1))
    )
    p_df = pd.DataFrame(p_arr, columns=[SB.iter] + columns)
    p_df[SB.iter] = p_df[SB.iter].astype(int) + 1

    p_df = pd.melt(
        p_df,
        id_vars=[SB.iter],
        value_vars=columns,
        var_name=getattr(SB, f"var_{idx}"),
        value_name=getattr(SB, f"val_{idx}"),
    )

    return p_df


log = logging.getLogger(__name__)
